@@ -55,13 +55,13 @@ holder_free(gobj_holder *holder)
 {
     if (holder->gobj) {
         if (!holder->destroyed) {
-            rbgobj_instance_call_cinfo_free(holder->gobj);
             g_object_set_qdata(holder->gobj, RUBY_GOBJECT_OBJ_KEY, NULL);
             g_object_weak_unref(holder->gobj, (GWeakNotify)weak_notify, holder);
+	    weak_notify(holder, holder->gobj);
         }
         holder->gobj = NULL;
     }
-    free(holder);
+    xfree(holder);
 }
 
 static VALUE
@@ -96,7 +96,6 @@ rbgobj_gobject_initialize(obj, cobj)
     gobj_holder* holder = g_object_get_qdata((GObject*)cobj, RUBY_GOBJECT_OBJ_KEY);
     if (holder)
         rb_raise(rb_eRuntimeError, "ruby wrapper for this GObject* already exists.");
-
     Data_Get_Struct(obj, gobj_holder, holder);
     holder->cinfo = RVAL2CINFO(obj);
     holder->gobj  = (GObject*)cobj;
@@ -119,16 +118,21 @@ rbgobj_gobject_initialize(obj, cobj)
 VALUE
 rbgobj_get_ruby_object_from_gobject(GObject* gobj, gboolean alloc)
 {
-    gobj_holder* holder = g_object_get_qdata(gobj, RUBY_GOBJECT_OBJ_KEY);
-    if (holder)
+    gobj_holder *holder;
+
+    holder = g_object_get_qdata(gobj, RUBY_GOBJECT_OBJ_KEY);
+    if (holder) {
         return holder->self;
-    else if (alloc) {
-        VALUE obj = gobj_s_allocate(GTYPE2CLASS(G_OBJECT_TYPE(gobj)));
+    } else if (alloc) {
+        VALUE obj;
+
+	obj = gobj_s_allocate(GTYPE2CLASS(G_OBJECT_TYPE(gobj)));
         gobj = g_object_ref(gobj);
         rbgobj_gobject_initialize(obj, (gpointer)gobj);
         return obj;
-    } else
+    } else {
         return Qnil;
+    }
 }
 
 GObject*
@@ -253,56 +257,50 @@ struct param_setup_arg {
     GParameter* params;
     guint param_size;
     VALUE params_hash;
+    guint index;
 };
 
-static VALUE
-_each_with_index(obj)
-    VALUE obj;
-{
-    rb_funcall(obj, rb_intern("each_with_index"), 0);
-    return Qnil;
-}
-
 static VALUE
 _params_setup(arg, param_setup_arg)
     VALUE arg;
     struct param_setup_arg* param_setup_arg;
 {
-    int n = NUM2INT(rb_ary_entry(arg, 1));
+    guint index;
     VALUE name, val;
     GParamSpec* pspec;
 
-    if (n >= param_setup_arg->param_size)
+    index = param_setup_arg->index;
+    if (index >= param_setup_arg->param_size)
        rb_raise(rb_eArgError, "too many parameters");
 
-    arg = rb_ary_entry(arg, 0);
-
     name = rb_ary_entry(arg, 0);
     val  = rb_ary_entry(arg, 1);
 
     if (SYMBOL_P(name))
-        param_setup_arg->params[n].name = rb_id2name(SYM2ID(name));
+        param_setup_arg->params[index].name = rb_id2name(SYM2ID(name));
     else
-        param_setup_arg->params[n].name = StringValuePtr(name);
+        param_setup_arg->params[index].name = StringValuePtr(name);
 
     pspec = g_object_class_find_property(
         param_setup_arg->gclass,
-        param_setup_arg->params[n].name);
+        param_setup_arg->params[index].name);
     if (!pspec)
         rb_raise(rb_eArgError, "No such property: %s",
-                 param_setup_arg->params[n].name);
+                 param_setup_arg->params[index].name);
 
-    g_value_init(&(param_setup_arg->params[n].value),
+    g_value_init(&(param_setup_arg->params[index].value),
                  G_PARAM_SPEC_VALUE_TYPE(pspec));
-    rbgobj_rvalue_to_gvalue(val, &(param_setup_arg->params[n].value));
-    
+    rbgobj_rvalue_to_gvalue(val, &(param_setup_arg->params[index].value));
+
+    param_setup_arg->index++;
+
     return Qnil;
 }
 
 static VALUE
 gobj_new_body(struct param_setup_arg* arg)
 {
-    rb_iterate((VALUE(*)_((VALUE)))_each_with_index, (VALUE)arg->params_hash, _params_setup, (VALUE)arg);
+    rb_iterate(rb_each, (VALUE)arg->params_hash, _params_setup, (VALUE)arg);
     return (VALUE)g_object_newv(G_TYPE_FROM_CLASS(arg->gclass),
                                 arg->param_size, arg->params);
 }
@@ -344,6 +342,7 @@ rbgobj_gobject_new(gtype, params_hash)
         arg.params = ALLOCA_N(GParameter, param_size);
         memset(arg.params, 0, sizeof(GParameter) * param_size);
         arg.params_hash = params_hash;
+        arg.index = 0;
 
         result = (GObject*)rb_ensure(&gobj_new_body, (VALUE)&arg,
                                      &gobj_new_ensure, (VALUE)&arg);
@@ -871,6 +870,7 @@ Init_gobject_gobject()
     rb_define_method(cGObject, "set_property", gobj_set_property, 2);
     rb_define_method(cGObject, "get_property", gobj_get_property, 1);
     rb_define_method(cGObject, "freeze_notify", gobj_freeze_notify, 0);
+    rb_undef_method(cGObject, "notify");
     rb_define_method(cGObject, "notify", gobj_notify, 1);
     rb_define_method(cGObject, "thaw_notify", gobj_thaw_notify, 0);
     rb_define_method(cGObject, "destroyed?", gobj_is_destroyed, 0);
