@@ -3,15 +3,16 @@
 
   rbgobj_signal.c -
 
-  $Author: sakai $
-  $Date: 2003/11/15 06:07:33 $
+  $Author: ssimons $
+  $Date: 2007/07/25 17:04:28 $
   created at: Sat Jul 27 16:56:01 JST 2002
 
+  Copyright (C) 2002-2004  Ruby-GNOME2 Project Team
   Copyright (C) 2002,2003  Masahiro Sakai
 
 **********************************************************************/
 
-#include "global.h"
+#include "rbgprivate.h"
 
 static VALUE cSignal;
 VALUE rbgobj_signal_wrap(guint sig_id);
@@ -45,7 +46,7 @@ rbgobj_get_signal_func(guint signal_id)
 
 /**********************************************************************/
 
-VALUE eNoSignalError;
+static VALUE eNoSignalError;
 
 // FIXME: use rb_protect
 static gboolean
@@ -64,7 +65,7 @@ accumulator_func(GSignalInvocationHint* ihint,
     tmp = rb_funcall(proc, rb_intern("call"), 3, hint, val, new);
     /* FIXME */
     if (TYPE(tmp) == T_ARRAY) {
-        continue_emission = RTEST(rb_ary_entry(tmp, 0));
+        continue_emission = RVAL2CBOOL(rb_ary_entry(tmp, 0));
         val = rb_ary_entry(tmp, 1);
     } else {
         val = tmp;        
@@ -113,16 +114,17 @@ gobj_s_signal_new(int argc, VALUE* argv, VALUE self)
         proc = rb_funcall(factory, rb_intern("call"), 2, self, ID2SYM(method_id));
 
         class_closure = g_rclosure_new(proc, Qnil, NULL);
+        g_rclosure_attach(class_closure, self);
     }
 
     if (NIL_P(params)) {
         n_params = 0;
         param_types = NULL;
     } else {
-        n_params = RARRAY(params)->len;
+        n_params = RARRAY_LEN(params);
         param_types = ALLOCA_N(GType, n_params);
         for (i = 0; i < n_params; i++)
-            param_types[i] = rbgobj_gtype_get(RARRAY(params)->ptr[i]);
+            param_types[i] = rbgobj_gtype_get(RARRAY_PTR(params)[i]);
     }
 
     sig = g_signal_newv(StringValuePtr(signal_name),
@@ -145,13 +147,14 @@ gobj_s_signal_new(int argc, VALUE* argv, VALUE self)
     return rbgobj_signal_wrap(sig);
 }
 
-void
+static void
 _signal_list(VALUE result, GType gtype)
 {
     guint n_ids, i;
     guint* ids = g_signal_list_ids(gtype, &n_ids);
     for (i = 0; i < n_ids; i++)
         rb_ary_push(result, rb_str_new2(g_signal_name(ids[i])));
+    g_free(ids);
 }
 
 static VALUE
@@ -165,7 +168,7 @@ gobj_s_signals(int argc, VALUE* argv, VALUE self)
     gtype = CLASS2GTYPE(self);
     result = rb_ary_new();
 
-    if (RTEST(inherited_too)){
+    if (RVAL2CBOOL(inherited_too)){
         guint n_interfaces, i;
         GType* interfaces = g_type_interfaces(gtype, &n_interfaces);
         for (i = 0; i < n_interfaces; i++)
@@ -187,12 +190,10 @@ gobj_s_signal(VALUE self, VALUE name)
     const char* sig_name;
     guint sig_id;
 
-    if (SYMBOL_P(name)){
+    if (SYMBOL_P(name))
         sig_name = rb_id2name(SYM2ID(name));
-    } else {
-        StringValue(name);
+    else
         sig_name = StringValuePtr(name);
-    }
     
     sig_id = g_signal_lookup(sig_name, CLASS2GTYPE(self));
     if (!sig_id)
@@ -214,19 +215,61 @@ gobj_sig_has_handler_pending(argc, argv, self)
 
     rb_scan_args(argc, argv, "11", &sig, &may_be_blocked);
 
-    if (SYMBOL_P(sig)){
+    if (SYMBOL_P(sig))
         sig_name = rb_id2name(SYM2ID(sig));
-    } else {
-        StringValue(sig);
+    else
         sig_name = StringValuePtr(sig);
-    }
 
     if (!g_signal_parse_name(sig_name, CLASS2GTYPE(CLASS_OF(self)), &signal_id, &detail, TRUE))
         rb_raise(eNoSignalError, "no such signal: %s", sig_name);
 
-    return g_signal_has_handler_pending(RVAL2GOBJ(self),
-                                        signal_id, detail,
-                                        RTEST(may_be_blocked)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(g_signal_has_handler_pending(RVAL2GOBJ(self),
+                                                   signal_id, detail,
+                                                   RVAL2CBOOL(may_be_blocked)));
+}
+
+static VALUE
+gobj_sig_connect_impl(after, argc, argv, self)
+    gboolean after;
+    int argc;
+    VALUE *argv;
+    VALUE self;
+{
+    VALUE sig, rest;
+    int i;
+    GClosure* rclosure;
+    const char* sig_name;
+    guint signal_id;
+    GQuark detail;
+    VALUE func;
+    GObject *g_object;
+    gchar *tag;
+
+    rb_scan_args(argc, argv, "1*", &sig, &rest);
+
+    if (NIL_P(rest)) rest = rb_ary_new();
+
+    if (SYMBOL_P(sig))
+        sig_name = rb_id2name(SYM2ID(sig));
+    else
+        sig_name = StringValuePtr(sig);
+
+    if (!g_signal_parse_name(sig_name, CLASS2GTYPE(CLASS_OF(self)), &signal_id, &detail, TRUE))
+        rb_raise(eNoSignalError, "no such signal: %s", sig_name);
+
+    func = rb_block_proc();
+    rclosure = g_rclosure_new(func, rest, 
+                              rbgobj_get_signal_func(signal_id));
+    g_rclosure_attach((GClosure *)rclosure, self);
+    g_object = RVAL2GOBJ(self);
+    tag = g_strdup_printf("%s::%s",
+                          G_OBJECT_CLASS_NAME(G_OBJECT_GET_CLASS(g_object)),
+                          sig_name);
+    g_rclosure_set_tag((GClosure *)rclosure, tag);
+    g_free(tag);
+    i = g_signal_connect_closure_by_id(g_object, signal_id, detail, rclosure, after);
+
+    return INT2FIX(i);
 }
 
 static VALUE
@@ -235,30 +278,7 @@ gobj_sig_connect(argc, argv, self)
     VALUE *argv;
     VALUE self;
 {
-    VALUE sig, rest;
-    int i;
-    GClosure* rclosure;
-    const char* sig_name;
-    guint signal_id;
-    GQuark detail;
-
-    rb_scan_args(argc, argv, "1*", &sig, &rest);
-
-    if (SYMBOL_P(sig)){
-        sig_name = rb_id2name(SYM2ID(sig));
-    } else {
-        StringValue(sig);
-        sig_name = StringValuePtr(sig);
-    }
-
-    if (!g_signal_parse_name(sig_name, CLASS2GTYPE(CLASS_OF(self)), &signal_id, &detail, TRUE))
-        rb_raise(eNoSignalError, "no such signal: %s", sig_name);
-
-    rclosure = g_rclosure_new(G_BLOCK_PROC(), rest, 
-                              rbgobj_get_signal_func(signal_id));
-    i = g_signal_connect_closure_by_id(RVAL2GOBJ(self), signal_id, detail, rclosure, FALSE);
-
-    return INT2FIX(i);
+    return gobj_sig_connect_impl(FALSE, argc, argv, self);
 }
 
 static VALUE
@@ -267,30 +287,7 @@ gobj_sig_connect_after(argc, argv, self)
     VALUE *argv;
     VALUE self;
 {
-    VALUE sig, rest;
-    int i;
-    GClosure* rclosure;
-    const char* sig_name;
-    guint signal_id;
-    GQuark detail;
-
-    rb_scan_args(argc, argv, "1*", &sig, &rest);
-
-    if (SYMBOL_P(sig))
-        sig_name = rb_id2name(SYM2ID(sig));
-    else {
-        StringValue(sig);
-        sig_name = StringValuePtr(sig);
-    }
-
-    if (!g_signal_parse_name(sig_name, CLASS2GTYPE(CLASS_OF(self)), &signal_id, &detail, TRUE))
-        rb_raise(eNoSignalError, "no such signal: %s", sig_name);
-
-    rclosure = g_rclosure_new(G_BLOCK_PROC(), rest, 
-                              rbgobj_get_signal_func(signal_id));
-    i = g_signal_connect_closure_by_id(RVAL2GOBJ(self), signal_id, detail, rclosure, TRUE);
-
-    return INT2FIX(i);
+    return gobj_sig_connect_impl(TRUE, argc, argv, self);
 }
 
 #if 0
@@ -319,17 +316,23 @@ struct emit_arg{
 static VALUE
 emit_body(struct emit_arg* arg)
 {
-    g_value_init(arg->instance_and_params->values,
-                 G_TYPE_FROM_INSTANCE(RVAL2GOBJ(arg->self)));
-    rbgobj_rvalue_to_gvalue(arg->self, arg->instance_and_params->values);
+    GValue param = { 0, };
+
+    g_value_init(&param, G_TYPE_FROM_INSTANCE(RVAL2GOBJ(arg->self)));
+    rbgobj_rvalue_to_gvalue(arg->self, &param);
+    g_value_array_append(arg->instance_and_params, &param);
+    g_value_unset(&param);
 
     {
-        GValue* params = arg->instance_and_params->values + 1;
         int i;
         for (i = 0; i < arg->query.n_params; i++){
             GType gtype = arg->query.param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE;
-            g_value_init(params + i, gtype);
-            rbgobj_rvalue_to_gvalue(rb_ary_entry(arg->args, i), params + i);
+
+            g_value_init(&param,  gtype);
+
+            rbgobj_rvalue_to_gvalue(rb_ary_entry(arg->args, i), &param);
+            g_value_array_append(arg->instance_and_params, &param);
+            g_value_unset(&param);
         }
     }
 
@@ -377,10 +380,8 @@ gobj_sig_emit(argc, argv, self)
 
     if (SYMBOL_P(sig))
         sig_name = rb_id2name(SYM2ID(sig));
-    else {
-        StringValue(sig);
+    else
         sig_name = StringValuePtr(sig);
-    }
 
     if (!g_signal_parse_name(sig_name,
                              CLASS2GTYPE(CLASS_OF(self)),
@@ -389,9 +390,9 @@ gobj_sig_emit(argc, argv, self)
 
     g_signal_query(signal_id, &arg.query);
 
-    if (arg.query.n_params != RARRAY(arg.args)->len)
-        rb_raise(rb_eArgError, "wrong number of arguments(%d for %d)",
-                 RARRAY(arg.args)->len + 1,
+    if (arg.query.n_params != RARRAY_LEN(arg.args))
+        rb_raise(rb_eArgError, "wrong number of arguments(%ld for %d)",
+                 RARRAY_LEN(arg.args) + 1,
                  arg.query.n_params + 1);
 
     arg.self = self;
@@ -411,10 +412,8 @@ gobj_sig_emit_stop(self, sig)
 
     if (SYMBOL_P(sig))
         sig_name = rb_id2name(SYM2ID(sig));
-    else {
-        StringValue(sig);
+    else
         sig_name = StringValuePtr(sig);
-    }
 
     if (!g_signal_parse_name(sig_name,
                              CLASS2GTYPE(CLASS_OF(self)),
@@ -431,8 +430,8 @@ static VALUE
 _sig_handler_block_ensure(arg)
     VALUE arg;
 {
-    VALUE self = RARRAY(arg)->ptr[0];
-    VALUE id   = RARRAY(arg)->ptr[1];
+    VALUE self = RARRAY_PTR(arg)[0];
+    VALUE id   = RARRAY_PTR(arg)[1];
     gobj_sig_handler_unblock(self, id);
     return Qnil;
 }
@@ -468,7 +467,7 @@ static VALUE
 gobj_sig_handler_is_connected(self, id)
      VALUE self, id;
 {
-    return g_signal_handler_is_connected(RVAL2GOBJ(self), NUM2ULONG(id)) ? Qtrue : Qfalse;
+    return CBOOL2RVAL(g_signal_handler_is_connected(RVAL2GOBJ(self), NUM2ULONG(id)));
 }
 
 #if 0
@@ -600,6 +599,7 @@ gobj_s_method_added(klass, id)
         VALUE proc = rb_funcall(f, rb_intern("call"), 2, klass, id);
         GClosure* rclosure = g_rclosure_new(proc, Qnil,
                                             rbgobj_get_signal_func(signal_id));
+        g_rclosure_attach((GClosure *)rclosure, klass);
         g_signal_override_class_closure(signal_id, cinfo->gtype, rclosure);
     }
 
@@ -764,7 +764,7 @@ query_inspect(self)
     { \
         GSignalQuery* query; \
         Data_Get_Struct(self, GSignalQuery, query); \
-        return (query->signal_flags & flag) ? Qtrue : Qfalse; \
+        return CBOOL2RVAL(query->signal_flags & flag); \
     }
 
 query_is_flag(G_SIGNAL_RUN_FIRST)
@@ -833,6 +833,7 @@ signal_add_emission_hook(int argc, VALUE* argv, VALUE self)
 
     closure = g_rclosure_new(proc, Qnil,
                              rbgobj_get_signal_func(query->signal_id));
+    g_rclosure_attach(closure, self);
     hook_id = g_signal_add_emission_hook_closure(query->signal_id, detail, closure);
     return ULONG2NUM(hook_id);
 }
@@ -849,6 +850,8 @@ signal_remove_emission_hook(VALUE self, VALUE hook_id)
 static void
 Init_signal_class()
 {
+    VALUE cSignalFlags, cSignalMatchType;
+
     cSignal = rb_define_class_under(mGLib, "Signal", rb_cData);
 
     rb_define_method(cSignal, "id", query_signal_id, 0);
@@ -864,15 +867,10 @@ Init_signal_class()
     rb_define_method(cSignal, "remove_emission_hook", signal_remove_emission_hook, 1);
 
     /* GSignalFlags */
-    rb_define_const(cSignal, "RUN_FIRST",   INT2FIX(G_SIGNAL_RUN_FIRST));
-    rb_define_const(cSignal, "RUN_LAST",    INT2FIX(G_SIGNAL_RUN_LAST));
-    rb_define_const(cSignal, "RUN_CLEANUP", INT2FIX(G_SIGNAL_RUN_CLEANUP));
-    rb_define_const(cSignal, "NO_RECURSE",  INT2FIX(G_SIGNAL_NO_RECURSE));
-    rb_define_const(cSignal, "DETAILED",    INT2FIX(G_SIGNAL_DETAILED));
-    rb_define_const(cSignal, "ACTION",      INT2FIX(G_SIGNAL_ACTION));
-    rb_define_const(cSignal, "NO_HOOKS",    INT2FIX(G_SIGNAL_NO_HOOKS));
-
-    rb_define_const(cSignal, "FLAGS_MASK",  INT2FIX(G_SIGNAL_FLAGS_MASK));
+    cSignalFlags = G_DEF_CLASS(G_TYPE_SIGNAL_FLAGS, "SignalFlags", mGLib);
+    G_DEF_CONSTANTS(cSignal, G_TYPE_SIGNAL_FLAGS, "G_SIGNAL_");
+    rb_define_const(cSignalFlags, "MASK", INT2NUM(G_SIGNAL_FLAGS_MASK));
+    rb_define_const(cSignal, "FLAGS_MASK", INT2NUM(G_SIGNAL_FLAGS_MASK));
 
     rb_define_method(cSignal, "run_first?", query_is_G_SIGNAL_RUN_FIRST, 0);
     rb_define_method(cSignal, "run_last?", query_is_G_SIGNAL_RUN_LAST, 0);
@@ -884,18 +882,16 @@ Init_signal_class()
 
 
     /* GConnectFlags */
-    rb_define_const(cSignal, "CONNECT_AFTER",   INT2FIX(G_CONNECT_AFTER));
-    rb_define_const(cSignal, "CONNECT_SWAPPED", INT2FIX(G_CONNECT_SWAPPED));
+    G_DEF_CLASS(G_TYPE_CONNECT_FLAGS, "ConnectFlags", mGLib);
+    G_DEF_CONSTANTS(cSignal, G_TYPE_CONNECT_FLAGS, "G_");
 
     /* GSignalMatchType */
-    rb_define_const(cSignal, "MATCH_ID",        INT2FIX(G_SIGNAL_MATCH_ID));
-    rb_define_const(cSignal, "MATCH_DETAIL",    INT2FIX(G_SIGNAL_MATCH_DETAIL));
-    rb_define_const(cSignal, "MATCH_CLOSURE",   INT2FIX(G_SIGNAL_MATCH_CLOSURE));
-    rb_define_const(cSignal, "MATCH_FUNC",      INT2FIX(G_SIGNAL_MATCH_FUNC));
-    rb_define_const(cSignal, "MATCH_DATA",      INT2FIX(G_SIGNAL_MATCH_DATA));
-    rb_define_const(cSignal, "MATCH_UNBLOCKED", INT2FIX(G_SIGNAL_MATCH_UNBLOCKED));
+    cSignalMatchType = G_DEF_CLASS(G_TYPE_SIGNAL_MATCH_TYPE,
+				   "SignalMatchType", mGLib);
+    G_DEF_CONSTANTS(cSignal, G_TYPE_SIGNAL_MATCH_TYPE, "G_SIGNAL_");
+    rb_define_const(cSignalMatchType, "MASK", INT2NUM(G_SIGNAL_MATCH_MASK));
+    rb_define_const(cSignal, "MATCH_MASK", INT2NUM(G_SIGNAL_MATCH_MASK));
 
-    rb_define_const(cSignal, "MATCH_MASK", INT2FIX(G_SIGNAL_MATCH_MASK));
     rb_define_const(cSignal, "TYPE_STATIC_SCOPE", INT2FIX(G_SIGNAL_TYPE_STATIC_SCOPE));
 
     eNoSignalError = rb_define_class_under(mGLib, "NoSignalError", rb_eNameError);
@@ -907,7 +903,7 @@ void
 rbgobj_define_action_methods(VALUE klass)
 {
     GType gtype = CLASS2GTYPE(klass);
-    GString* source = g_string_new(NULL);
+    GString* source;
     guint n_ids;
     guint* ids;
     int i;
@@ -916,7 +912,10 @@ rbgobj_define_action_methods(VALUE klass)
         return;
 
     ids = g_signal_list_ids(gtype, &n_ids);
+    if (n_ids == 0)
+        return;
 
+    source = g_string_new(NULL);
     for (i = 0; i < n_ids; i++){
         GSignalQuery query;
         g_signal_query(ids[i], &query);
@@ -948,7 +947,9 @@ rbgobj_define_action_methods(VALUE klass)
         }
     }
 
-    rb_funcall(klass, rb_intern("module_eval"), 1, rb_str_new2(source->str));
+    if (source->len > 0)
+        rb_funcall(klass, rb_intern("module_eval"), 1, rb_str_new2(source->str));
+    g_string_free(source, TRUE);
 }
 
 /**********************************************************************/
