@@ -3,8 +3,8 @@
 
   rbgdkselection.c -
 
-  $Author: mutoh $
-  $Date: 2004/03/05 16:24:30 $
+  $Author: ggc $
+  $Date: 2007/07/13 16:07:31 $
 
   Copyright (C) 2002-2004 Masao Mutoh
 ************************************************/
@@ -27,7 +27,7 @@ gdkselection_owner_set(argc, argv, self)
         rb_scan_args(argc, argv, "40", &owner, &selection, &time, &send_event);
         ret = gdk_selection_owner_set(GDK_WINDOW(RVAL2GOBJ(owner)), 
                                       RVAL2ATOM(selection), 
-                                      NUM2UINT(time), RTEST(send_event));
+                                      NUM2UINT(time), RVAL2CBOOL(send_event));
     } else {
 #if GTK_CHECK_VERSION(2,2,0)
       VALUE display = Qnil;
@@ -35,13 +35,13 @@ gdkselection_owner_set(argc, argv, self)
       ret = gdk_selection_owner_set_for_display(GDK_DISPLAY_OBJECT(RVAL2GOBJ(display)),
                                                 GDK_WINDOW(RVAL2GOBJ(owner)), 
                                                 RVAL2ATOM(selection), 
-                                                NUM2UINT(time), RTEST(send_event));
+                                                NUM2UINT(time), RVAL2CBOOL(send_event));
 #else
       rb_raise(rb_eArgError, "Wrong number of arguments: %d", argc);
 #endif
 
     }
-    return ret ? Qtrue : Qfalse;
+    return CBOOL2RVAL(ret);
 }
 
 static VALUE
@@ -84,11 +84,15 @@ gdkselection_property_get(self, requestor)
     guchar *data;
     GdkAtom prop_type;
     gint prop_format;
+    VALUE ary;
 
     gdk_selection_property_get(GDK_WINDOW(RVAL2GOBJ(requestor)), &data, 
                                &prop_type, &prop_format);
-    return rb_ary_new3(3, rb_str_new2(data), GATOM2RVAL(prop_type), 
-                       INT2NUM(prop_format));
+
+    ary = rb_ary_new3(3, CSTR2RVAL((const char*)data), GATOM2RVAL(prop_type), 
+                      INT2NUM(prop_format));
+    g_free(data);
+    return ary;
 }
 
 static VALUE
